@@ -157,7 +157,7 @@ CPLErr GDALEXRRasterBand::IReadBlock(int nBlockXOff, int nBlockYOff,
     {
         FrameBuffer fb;
         const size_t sizeOfElt = sizeof(float); // sizeof(uint32) as well
-        const auto slice = 
+        const auto slice =
             Slice(eDataType == GDT_Float32 ? FLOAT : UINT,
                     static_cast<char*>(pImage) -
                     (poGDS->m_nDWMinX + nBlockXOff * nBlockXSize +
@@ -449,6 +449,12 @@ class GDALEXRIOStreamException final: public std::exception
         const char* what() const noexcept override { return m_msg.c_str(); }
 };
 
+#if OPENEXR_VERSION_MAJOR < 3
+typedef Int64 IoInt64Type;
+#else
+typedef uint64_t IoInt64Type;
+#endif
+
 class GDALEXRIOStream final: public IStream, public OStream
 {
   public:
@@ -459,10 +465,10 @@ class GDALEXRIOStream final: public IStream, public OStream
 
     virtual bool        read (char c[/*n*/], int n) override;
     virtual void        write (const char c[/*n*/], int n) override;
-    virtual Int64       tellg () override;
-    virtual Int64       tellp () override { return tellg(); }
-    virtual void        seekg (Int64 pos) override;
-    virtual void        seekp (Int64 pos) override { return seekg(pos); }
+    virtual IoInt64Type tellg () override;
+    virtual IoInt64Type tellp () override { return tellg(); }
+    virtual void        seekg (IoInt64Type pos) override;
+    virtual void        seekp (IoInt64Type pos) override { return seekg(pos); }
 
   private:
     VSILFILE* m_fp;
@@ -492,12 +498,12 @@ void GDALEXRIOStream::write (const char c[/*n*/], int n)
     }
 }
 
-Int64 GDALEXRIOStream::tellg ()
+IoInt64Type GDALEXRIOStream::tellg ()
 {
-    return static_cast<Int64>(VSIFTellL(m_fp));
+    return static_cast<IoInt64Type>(VSIFTellL(m_fp));
 }
 
-void GDALEXRIOStream::seekg (Int64 pos)
+void GDALEXRIOStream::seekg (IoInt64Type pos)
 {
     VSIFSeekL(m_fp, static_cast<vsi_l_offset>(pos), SEEK_SET);
 }
@@ -562,7 +568,7 @@ GDALDataset* GDALEXRDataset::Open(GDALOpenInfo* poOpenInfo)
 
     try
     {
-        auto poDS = std::unique_ptr<GDALEXRDataset>(new GDALEXRDataset());
+        auto poDS = cpl::make_unique<GDALEXRDataset>();
         poDS->m_pIStream.reset(new GDALEXRIOStream(fp, osFilename));
         poDS->m_pMPIF.reset(new MultiPartInputFile(*poDS->m_pIStream));
         if( iPart > 0 && iPart > poDS->m_pMPIF->parts() )
@@ -730,8 +736,7 @@ GDALDataset* GDALEXRDataset::Open(GDALOpenInfo* poOpenInfo)
                     {
                         break;
                     }
-                    auto poOvrDS =
-                        std::unique_ptr<GDALEXRDataset>(new GDALEXRDataset());
+                    auto poOvrDS = cpl::make_unique<GDALEXRDataset>();
                     // coverity[escape]
                     poOvrDS->m_poParent = poDS.get();
                     poOvrDS->m_iLevel = iLevel;
@@ -759,7 +764,7 @@ GDALDataset* GDALEXRDataset::Open(GDALOpenInfo* poOpenInfo)
                 const Attribute *attr = &iter.attribute();
                 const StringAttribute *stringAttr =
                                 dynamic_cast <const StringAttribute *>(attr);
-                const M33dAttribute* m33DAttr = 
+                const M33dAttribute* m33DAttr =
                                 dynamic_cast <const M33dAttribute *>(attr);
                 if ( stringAttr && strcmp(iter.name(), "gdal:crsWkt") == 0)
                 {
@@ -1015,7 +1020,7 @@ GDALDataset *GDALEXRDataset::CreateCopy( const char* pszFilename,
         if( bPreview )
         {
             const int previewWidth = 100;
-            const int previewHeight = std::max(1, 
+            const int previewHeight = std::max(1,
                 static_cast<int>(static_cast<GIntBig>(previewWidth) * nYSize / nXSize));
             std::vector<PreviewRgba> pixels(previewWidth * previewHeight);
             if( poSrcDS->RasterIO(
@@ -1191,7 +1196,7 @@ GDALDataset *GDALEXRDataset::CreateCopy( const char* pszFilename,
                         FrameBuffer fb;
                         for( int iBand = 0; iBand < nBands; iBand++ )
                         {
-                            const auto slice = 
+                            const auto slice =
                                 Slice(pixelType,
                                     sliceBuffer +
                                         iBand * pixelTypeSize * nChunkXSize * nChunkYSize -
@@ -1351,7 +1356,7 @@ GDALDataset *GDALEXRDataset::CreateCopy( const char* pszFilename,
                 const int nLinesToRead = std::min(nChunkYSize, nYSize - y);
                 for( int iBand = 0; iBand < nBands; iBand++ )
                 {
-                    const auto slice = 
+                    const auto slice =
                         Slice(pixelType,
                             sliceBuffer +
                                 iBand * pixelTypeSize * nXSize * nLinesToRead -
@@ -1478,7 +1483,7 @@ public:
 GDALEXRWritableDataset::~GDALEXRWritableDataset()
 {
     WriteHeader();
-    FlushCache();
+    FlushCache(true);
 }
 
 /************************************************************************/
@@ -1831,7 +1836,7 @@ CPLErr GDALEXRWritableRasterBand::IWriteBlock(int nBlockXOff,
         {
             char* const dstPtr = poGDS->m_pSliceBuffer +
                 iBand * poGDS->m_nBufferEltSize * nPixelsInBlock;
-            const auto slice = 
+            const auto slice =
                 Slice(poGDS->m_pixelType,
                       dstPtr -
                         (x * poGDS->m_nBufferEltSize +
@@ -1841,7 +1846,7 @@ CPLErr GDALEXRWritableRasterBand::IWriteBlock(int nBlockXOff,
             fb.insert(poGDS->m_channelNames[iBand], slice);
 
             const void* srcPtr = nullptr;
-            if( iBand+1 == nBand) 
+            if( iBand+1 == nBand)
                 srcPtr = pImage;
             else if( apoBlocks[iBand] )
                 srcPtr = apoBlocks[iBand]->GetDataRef();
