@@ -323,6 +323,8 @@ public:
 
     bool IsWritable() const override { return !m_poShared->IsReadOnly(); }
 
+    const std::string& GetFilename() const override { return m_poShared->GetFilename(); }
+
     const std::vector<std::shared_ptr<GDALDimension>>& GetDimensions() const override { return m_dims; }
 
     const GDALExtendedDataType &GetDataType() const override { return m_dt; }
@@ -450,6 +452,16 @@ public:
     const GDALExtendedDataType &GetDataType() const override { return m_dt; }
 };
 
+/************************************************************************/
+/*                        HDF5SharedResources()                         */
+/************************************************************************/
+
+HDF5SharedResources::HDF5SharedResources(const std::string& osFilename):
+    m_osFilename(osFilename),
+    m_poPAM(std::make_shared<GDALPamMultiDim>(osFilename))
+{
+}
+
 /************************************************************************/
 /*                        ~HDF5SharedResources()                        */
 /************************************************************************/
@@ -1470,12 +1482,44 @@ static void FreeDynamicMemory(GByte* pabyPtr, hid_t hDataType)
     }
 }
 
+/************************************************************************/
+/*                   CreateMapTargetComponentsToSrc()                   */
+/************************************************************************/
+
+static std::vector<unsigned> CreateMapTargetComponentsToSrc(
+                hid_t hSrcDataType, const GDALExtendedDataType& dstDataType)
+{
+    CPLAssert( H5Tget_class(hSrcDataType) == H5T_COMPOUND );
+    CPLAssert( dstDataType.GetClass() == GEDTC_COMPOUND );
+
+    const unsigned nMembers = H5Tget_nmembers(hSrcDataType);
+    std::map<std::string, unsigned> oMapSrcCompNameToIdx;
+    for(unsigned i = 0; i < nMembers; i++ )
+    {
+        char* pszName = H5Tget_member_name(hSrcDataType, i);
+        if( pszName )
+            oMapSrcCompNameToIdx[pszName] = i;
+    }
+
+    std::vector<unsigned> ret;
+    const auto& comps = dstDataType.GetComponents();
+    ret.reserve(comps.size());
+    for( const auto& comp: comps )
+    {
+        auto oIter = oMapSrcCompNameToIdx.find(comp->GetName());
+        CPLAssert( oIter != oMapSrcCompNameToIdx.end() );
+        ret.emplace_back(oIter->second);
+    }
+    return ret;
+}
+
 /************************************************************************/
 /*                            CopyValue()                               */
 /************************************************************************/
 
 static void CopyValue(const GByte* pabySrcBuffer, hid_t hSrcDataType,
-                      GByte* pabyDstBuffer, const GDALExtendedDataType& dstDataType)
+                      GByte* pabyDstBuffer, const GDALExtendedDataType& dstDataType,
+                      const std::vector<unsigned>& mapDstCompsToSrcComps)
 {
     const auto klass = H5Tget_class(hSrcDataType);
     if( klass == H5T_STRING )
@@ -1503,7 +1547,6 @@ static void CopyValue(const GByte* pabySrcBuffer, hid_t hSrcDataType,
     }
     else if( klass == H5T_COMPOUND )
     {
-        const unsigned nMembers = H5Tget_nmembers(hSrcDataType);
         if( dstDataType.GetClass() != GEDTC_COMPOUND )
         {
             // Typically source is complex data type
@@ -1519,14 +1562,16 @@ static void CopyValue(const GByte* pabySrcBuffer, hid_t hSrcDataType,
         else
         {
             const auto& comps = dstDataType.GetComponents();
-            CPLAssert( nMembers == comps.size() );
-            for( unsigned i = 0; i < nMembers; i++ )
+            CPLAssert( comps.size() == mapDstCompsToSrcComps.size() );
+            const std::vector<unsigned> empty;
+            for( size_t iDst = 0; iDst < comps.size(); ++iDst )
             {
-                auto hMemberType = H5Tget_member_type(hSrcDataType, i);
-                CopyValue( pabySrcBuffer + H5Tget_member_offset(hSrcDataType, i),
-                        hMemberType,
-                        pabyDstBuffer + comps[i]->GetOffset(),
-                        comps[i]->GetType() );
+                const unsigned iSrc = mapDstCompsToSrcComps[iDst];
+                auto hMemberType = H5Tget_member_type(hSrcDataType, iSrc);
+                CopyValue( pabySrcBuffer + H5Tget_member_offset(hSrcDataType, iSrc),
+                           hMemberType,
+                           pabyDstBuffer + comps[iDst]->GetOffset(),
+                           comps[iDst]->GetType(), empty );
                 H5Tclose(hMemberType);
             }
         }
@@ -1534,7 +1579,7 @@ static void CopyValue(const GByte* pabySrcBuffer, hid_t hSrcDataType,
     else if( klass == H5T_ENUM )
     {
         auto hParent = H5Tget_super(hSrcDataType);
-        CopyValue(pabySrcBuffer, hParent, pabyDstBuffer, dstDataType);
+        CopyValue(pabySrcBuffer, hParent, pabyDstBuffer, dstDataType, {});
         H5Tclose(hParent);
     }
     else
@@ -1591,12 +1636,19 @@ static void CopyToFinalBuffer(void* pDstBuffer,
     const GByte* pabySrcBuffer = static_cast<const GByte*>(pTemp);
     pabyDstBufferStack[0] = static_cast<GByte*>(pDstBuffer);
     size_t iDim = 0;
+    const auto mapDstCompsToSrcComps =
+        (H5Tget_class(hSrcDataType) == H5T_COMPOUND &&
+         bufferDataType.GetClass() == GEDTC_COMPOUND) ?
+        CreateMapTargetComponentsToSrc(hSrcDataType, bufferDataType) :
+        std::vector<unsigned>();
+
 lbl_next_depth:
     if( iDim == nDims )
     {
         CopyValue(
             pabySrcBuffer, hSrcDataType,
-            pabyDstBufferStack[nDims], bufferDataType);
+            pabyDstBufferStack[nDims], bufferDataType,
+            mapDstCompsToSrcComps);
     }
     else
     {
@@ -1852,6 +1904,12 @@ static void CopyAllAttrValuesInto(size_t nDims,
     std::vector<size_t> anStackCount(nDims);
     std::vector<const GByte*> pabySrcBufferStack(nDims + 1);
     std::vector<GByte*> pabyDstBufferStack(nDims + 1);
+    const auto mapDstCompsToSrcComps =
+        (H5Tget_class(hSrcBufferType) == H5T_COMPOUND &&
+         bufferDataType.GetClass() == GEDTC_COMPOUND) ?
+        CreateMapTargetComponentsToSrc(hSrcBufferType, bufferDataType) :
+        std::vector<unsigned>();
+
     pabySrcBufferStack[0] = static_cast<const GByte*>(pabySrcBuffer);
     if( nDims > 0 )
         pabySrcBufferStack[0] += arrayStartIdx[0] * nSrcDataTypeSize;
@@ -1862,7 +1920,8 @@ lbl_next_depth:
     {
         CopyValue(
             pabySrcBufferStack[nDims], hSrcBufferType,
-            pabyDstBufferStack[nDims], bufferDataType);
+            pabyDstBufferStack[nDims], bufferDataType,
+            mapDstCompsToSrcComps);
     }
     else
     {
@@ -2070,7 +2129,7 @@ GDALDataset *HDF5Dataset::OpenMultiDim( GDALOpenInfo *poOpenInfo )
         return nullptr;
     }
 
-    auto poSharedResources = std::make_shared<GDAL::HDF5SharedResources>();
+    auto poSharedResources = std::make_shared<GDAL::HDF5SharedResources>(pszFilename);
     poSharedResources->m_hHDF5 = hHDF5;
 
     auto poGroup(OpenGroup(poSharedResources));
