@@ -46,7 +46,7 @@ constexpr int RECORD_SIZE = 512;
 #include "ogr_spatialref.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: isis2dataset.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
+CPL_CVSID("$Id: isis2dataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -133,7 +133,7 @@ ISIS2Dataset::ISIS2Dataset() :
 ISIS2Dataset::~ISIS2Dataset()
 
 {
-    FlushCache();
+    FlushCache(true);
     if( fpImage != nullptr )
         VSIFCloseL( fpImage );
 }
