@@ -28,6 +28,9 @@
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  *  SOFTWARE.
  *******************************************************************************/
+
+#include "cpl_port.h"
+#include "cpl_safemaths.hpp"
 #include "cadheader.h"
 #include "opencad_api.h"
 #include "dwg/io.h"
@@ -186,25 +189,38 @@ void CADHandle::addOffset( unsigned char val )
 
 long CADHandle::getAsLong( const CADHandle& ref_handle ) const
 {
-    switch( code )
+    // FIXME: Remove GDAL specific code. The library cannot compile as separate project.
+    try
     {
-        case 0x06:
+        switch( code )
         {
-            return getAsLong(ref_handle.handleOrOffset) + 1;
-        }
-        case 0x08:
-        {
-            return getAsLong(ref_handle.handleOrOffset) - 1;
-        }
-        case 0x0A:
-        {
-            return getAsLong(ref_handle.handleOrOffset) + getAsLong(handleOrOffset);
-        }
-        case 0x0C:
-        {
-            return getAsLong(ref_handle.handleOrOffset) - getAsLong(handleOrOffset);
+            case 0x06:
+            {
+                return static_cast<long>((CPLSM(static_cast<GInt64>(getAsLong(ref_handle.handleOrOffset))) +
+                        CPLSM(static_cast<GInt64>(1))).v());
+            }
+            case 0x08:
+            {
+                return static_cast<long>((CPLSM(static_cast<GInt64>(getAsLong(ref_handle.handleOrOffset))) -
+                        CPLSM(static_cast<GInt64>(1))).v());
+            }
+            case 0x0A:
+            {
+                return static_cast<long>((CPLSM(static_cast<GInt64>(getAsLong(ref_handle.handleOrOffset))) +
+                        CPLSM(static_cast<GInt64>(getAsLong(handleOrOffset)))).v());
+            }
+            case 0x0C:
+            {
+                return static_cast<long>((CPLSM(static_cast<GInt64>(getAsLong(ref_handle.handleOrOffset))) -
+                        CPLSM(static_cast<GInt64>(getAsLong(handleOrOffset)))).v());
+            }
         }
     }
+    catch( const CPLSafeIntOverflow& )
+    {
+        // TODO: handle this differently ?
+        return 0;
+    }
 
     return getAsLong(handleOrOffset);
 }
@@ -216,18 +232,15 @@ long CADHandle::getAsLong() const
 
 long CADHandle::getAsLong(const std::vector<unsigned char>& handle)
 {
-    long result = 0;
+    unsigned long result = 0;
     if( handle.empty() )
         return result;
     size_t copySize = handle.size();
     if( copySize > sizeof(long) )
         copySize = sizeof(long);
-    memcpy( &result, handle.data(), copySize );
-    // NOTE: Second argument below was previously handle.size(). This was
-    // clearly wrong (consider the case where handle.size() > sizeof(long))
-    // but I don't know what the correct value should be
-    SwapEndianness( result, copySize );
-    return result;
+    for( size_t i = 0; i < copySize; ++i )
+        result = result * 256U + handle[i];
+    return static_cast<long>(result);
 }
 
 bool CADHandle::isNull() const
