@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2009, Frank Warmerdam <warmerdam@pobox.com>
- * Copyright (c) 2010-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2010-2013, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -33,7 +33,7 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: ogrdxfdatasource.cpp a977483f0946e73193ff13b713d93c3aef7ba22e 2019-01-18 11:44:31 +1100 Alan Thomas $")
+CPL_CVSID("$Id: ogrdxfdatasource.cpp 8d5b1df5a0e407130147947c7b88456c0659aa56 2021-06-03 19:13:55 +1000 Alan Thomas $")
 
 /************************************************************************/
 /*                          OGRDXFDataSource()                          */
@@ -413,7 +413,8 @@ bool OGRDXFDataSource::ReadLayerDefinition()
           case 62:
             oLayerProperties["Color"] = szLineBuf;
 
-            if( atoi(szLineBuf) < 0 ) // Is layer off?
+            // Is layer off?
+            if( atoi(szLineBuf) < 0 && oLayerProperties["Hidden"] != "2" )
                 oLayerProperties["Hidden"] = "1";
             break;
 
@@ -423,8 +424,10 @@ bool OGRDXFDataSource::ReadLayerDefinition()
 
           case 70:
             oLayerProperties["Flags"] = szLineBuf;
-            if( atoi(szLineBuf) & 0x01 ) // Is layer frozen?
-                oLayerProperties["Hidden"] = "1";
+
+            // Is layer frozen?
+            if( atoi(szLineBuf) & 0x01 )
+                oLayerProperties["Hidden"] = "2";
             break;
 
           case 370:
@@ -1002,7 +1005,7 @@ size_t OGRDXFDataSource::GetEntryFromAcDsDataSection(
     int iPrevOffset = oReader.iSrcBufferFileOffset + oReader.iSrcBufferOffset;
     int nPrevLineNumber = oReader.nLineNumber;
 
-    char szLineBuf[270]; // TODO figure out what to do with this re character escapes
+    char szLineBuf[4096];
     int nCode = 0;
     bool bFound = false;
 
