@@ -6,7 +6,7 @@
  *******************************************************************************
  *  The MIT License (MIT)
  *
- *  Copyright (c) 2018-2019, NextGIS
+ *  Copyright (c) 2018-2020, NextGIS <info@nextgis.com>
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a copy
  *  of this software and associated documentation files (the "Software"), to deal
@@ -53,7 +53,6 @@ public:
  * OGRNGWDataset()
  */
 OGRNGWDataset::OGRNGWDataset() :
-    bReadWrite(false),
     nBatchSize(-1),
     nPageSize(-1),
     bFetchedPermissions(false),
@@ -75,7 +74,7 @@ OGRNGWDataset::OGRNGWDataset() :
 OGRNGWDataset::~OGRNGWDataset()
 {
     // Last sync with server.
-    OGRNGWDataset::FlushCache();
+    OGRNGWDataset::FlushCache(true);
 
     if( poRasterDS != nullptr )
     {
@@ -100,12 +99,12 @@ void OGRNGWDataset::FetchPermissions()
         return;
     }
 
-    if( bReadWrite )
+    if( IsUpdateMode() )
     {
         // Check connection and is it read only.
         char **papszHTTPOptions = GetHeaders();
         stPermissions = NGWAPI::CheckPermissions( osUrl, osResourceId,
-            papszHTTPOptions, bReadWrite );
+            papszHTTPOptions, IsUpdateMode() );
         CSLDestroy( papszHTTPOptions );
     }
     else
@@ -175,7 +174,7 @@ bool OGRNGWDataset::Open( const std::string &osUrlIn,
     osUrl = osUrlIn;
     osResourceId = osResourceIdIn;
 
-    bReadWrite = bUpdateIn;
+    eAccess = bUpdateIn ? GA_Update : GA_ReadOnly;
 
     osUserPwd = CSLFetchNameValueDef( papszOpenOptionsIn, "USERPWD",
         CPLGetConfigOption("NGW_USERPWD", ""));
@@ -202,6 +201,14 @@ bool OGRNGWDataset::Open( const std::string &osUrlIn,
     osJsonDepth = CSLFetchNameValueDef( papszOpenOptionsIn, "JSON_DEPTH",
         CPLGetConfigOption("NGW_JSON_DEPTH", "32"));
 
+    osExtensions = CSLFetchNameValueDef(papszOpenOptionsIn, "EXTENSIONS",
+        CPLGetConfigOption("NGW_EXTENSIONS", ""));
+
+    if (osExtensions.empty())
+    {
+        bExtInNativeData = false;
+    }
+
     return Init( nOpenFlagsIn );
 }
 
@@ -268,7 +275,7 @@ bool OGRNGWDataset::Init(int nOpenFlagsIn)
             else if( (osResourceType == "vector_layer" ||
                 osResourceType == "postgis_layer") )
             {
-                // Cehck feature paging.
+                // Check feature paging.
                 FillCapabilities( papszHTTPOptions );
                 // Add vector layer.
                 AddLayer( oRoot, papszHTTPOptions, nOpenFlagsIn );
@@ -276,6 +283,7 @@ bool OGRNGWDataset::Init(int nOpenFlagsIn)
             else if( osResourceType == "mapserver_style" ||
                 osResourceType == "qgis_vector_style" ||
                 osResourceType == "raster_style" ||
+                osResourceType == "qgis_raster_style" ||
                 osResourceType == "wmsclient_layer" )
             {
                 // GetExtent from parent.
@@ -317,7 +325,8 @@ bool OGRNGWDataset::Init(int nOpenFlagsIn)
                         {
                             nEPSG = oParentRoot.GetInteger("vector_layer/srs/id", nEPSG);
                         }
-                        else if( osResourceType == "raster_style")
+                        else if( osResourceType == "raster_style" ||
+                                 osResourceType == "qgis_raster_style")
                         {
                             nEPSG = oParentRoot.GetInteger("raster_layer/srs/id", nEPSG);
                         }
@@ -498,6 +507,7 @@ void OGRNGWDataset::AddRaster( const CPLJSONObject &oRasterJsonObj,
     if( osResourceType == "mapserver_style" ||
         osResourceType == "qgis_vector_style" ||
         osResourceType == "raster_style" ||
+        osResourceType == "qgis_raster_style" ||
         osResourceType == "wmsclient_layer" )
     {
         osOutResourceId = oRasterJsonObj.GetString( "resource/id" );
@@ -517,7 +527,8 @@ void OGRNGWDataset::AddRaster( const CPLJSONObject &oRasterJsonObj,
             {
                 CPLJSONObject oChild = oChildren[i];
                 osResourceType = oChild.GetString("resource/cls");
-                if( osResourceType == "raster_style" )
+                if( osResourceType == "raster_style" ||
+                    osResourceType == "qgis_raster_style" )
                 {
                     AddRaster( oChild, papszOptions );
                 }
@@ -552,7 +563,7 @@ OGRLayer *OGRNGWDataset::ICreateLayer( const char *pszNameIn,
                                            OGRwkbGeometryType eGType,
                                            char **papszOptions )
 {
-    if( !bReadWrite )
+    if( !IsUpdateMode() )
     {
         CPLError(CE_Failure, CPLE_AppDefined,
             "Operation not available in read-only mode");
@@ -569,7 +580,8 @@ OGRLayer *OGRNGWDataset::ICreateLayer( const char *pszNameIn,
     }
 
     // Check input parameters.
-    if( eGType < wkbPoint || eGType > wkbMultiPolygon )
+    if( (eGType < wkbPoint || eGType > wkbMultiPolygon) &&
+        (eGType < wkbPoint25D || eGType > wkbMultiPolygon25D) )
     {
         CPLError(CE_Failure, CPLE_AppDefined,
             "Unsupported geometry type: %s", OGRGeometryTypeToName(eGType));
@@ -623,16 +635,11 @@ OGRLayer *OGRNGWDataset::ICreateLayer( const char *pszNameIn,
     // Create layer.
     std::string osKey = CSLFetchNameValueDef( papszOptions, "KEY", "");
     std::string osDesc = CSLFetchNameValueDef( papszOptions, "DESCRIPTION", "");
-    OGRSpatialReference* poSRSClone = poSpatialRef;
-    if( poSRSClone )
-    {
-        poSRSClone = poSRSClone->Clone();
-        poSRSClone->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
-    }
+    OGRSpatialReference* poSRSClone = poSpatialRef->Clone();
+    poSRSClone->SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     OGRNGWLayer *poLayer = new OGRNGWLayer( this, pszNameIn, poSRSClone, eGType,
         osKey, osDesc );
-    if( poSRSClone )
-        poSRSClone->Release();
+    poSRSClone->Release();
     papoLayers = (OGRNGWLayer**) CPLRealloc(papoLayers, (nLayers + 1) *
         sizeof(OGRNGWLayer*));
     papoLayers[nLayers++] = poLayer;
@@ -644,7 +651,7 @@ OGRLayer *OGRNGWDataset::ICreateLayer( const char *pszNameIn,
  */
 OGRErr OGRNGWDataset::DeleteLayer( int iLayer )
 {
-    if( !bReadWrite )
+    if( !IsUpdateMode() )
     {
         CPLError(CE_Failure, CPLE_AppDefined,
             "Operation not available in read-only mode.");
@@ -789,9 +796,9 @@ CPLErr OGRNGWDataset::SetMetadataItem( const char *pszName,
 /*
  * FlushCache()
  */
-void OGRNGWDataset::FlushCache()
+void OGRNGWDataset::FlushCache(bool bAtClosing)
 {
-    GDALDataset::FlushCache();
+    GDALDataset::FlushCache(bAtClosing);
     FlushMetadata( GetMetadata("NGW") );
 }
 
@@ -816,7 +823,7 @@ char **OGRNGWDataset::GetHeaders() const
 /*
  * SQLUnescape()
  * Get from gdal/ogr/ogrsf_frmts/sqlite/ogrsqliteutility.cpp as we don't want
- * depenency on sqlite
+ * dependency on sqlite
  */
 static CPLString SQLUnescape( const char *pszVal )
 {
@@ -844,7 +851,7 @@ static CPLString SQLUnescape( const char *pszVal )
 /*
  * SQLTokenize()
  * Get from gdal/ogr/ogrsf_frmts/sqlite/ogrsqliteutility.cpp as we don't want
- * depenency on sqlite
+ * dependency on sqlite
  */
 static char **SQLTokenize( const char *pszStr )
 {
@@ -1102,14 +1109,38 @@ OGRLayer *OGRNGWDataset::ExecuteSQL( const char *pszStatement,
             }
 
             std::string osNgwSelect;
+            for( int iKey = 0; iKey < oSelect.order_specs; iKey++ )
+            {
+                swq_order_def *psKeyDef = oSelect.order_defs + iKey;
+                if(iKey > 0 )
+                {
+                    osNgwSelect += ",";
+                }
+
+                if( psKeyDef->ascending_flag == TRUE )
+                {
+                    osNgwSelect += psKeyDef->field_name;
+                }
+                else
+                {
+                    osNgwSelect += "-" + std::string(psKeyDef->field_name);
+                }
+            }
+
             if( oSelect.where_expr != nullptr )
             {
-                osNgwSelect = OGRNGWLayer::TranslateSQLToFilter(
-                    oSelect.where_expr);
-                if( osNgwSelect.empty() )
+                if( !osNgwSelect.empty() )
                 {
-                    bSkip = true;
+                    osNgwSelect += "&";
                 }
+                osNgwSelect += OGRNGWLayer::TranslateSQLToFilter(
+                    oSelect.where_expr );
+
+            }
+
+            if( osNgwSelect.empty() )
+            {
+                bSkip = true;
             }
 
             if( !bSkip )
@@ -1239,3 +1270,12 @@ void OGRNGWDataset::FillCapabilities( char **papszOptions )
         }
     }
 }
+
+
+/*
+ * Extensions()
+ */
+std::string OGRNGWDataset::Extensions() const
+{
+    return osExtensions;
+}
