@@ -55,7 +55,7 @@
 #include "cpl_vsi_virtual.h"
 
 
-CPL_CVSID("$Id: cpl_vsil.cpp fa752ad6eabafaf630a704e1892a9d837d683cb3 2021-03-06 17:04:38 +0100 Even Rouault $")
+CPL_CVSID("$Id: cpl_vsil.cpp 5d5a9396da0e1c66349b4fdbe9d01a6a719eb1e9 2021-10-26 11:22:02 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                             VSIReadDir()                             */
@@ -135,7 +135,7 @@ char **VSIReadDirEx( const char *pszPath, int nMaxFiles )
  * @param pszFilename the path of a filename to inspect
  * UTF-8 encoded.
  * @return The list of entries, relative to the directory, of all sidecar
- * files available or NULL if the list is not known. 
+ * files available or NULL if the list is not known.
  * Filenames are returned in UTF-8 encoding.
  * Most implementations will return NULL, and a subsequent ReadDir will
  * list all files available in the file's directory. This function will be
@@ -155,15 +155,6 @@ char **VSISiblingFiles( const char *pszFilename)
 /*                             VSIReadRecursive()                       */
 /************************************************************************/
 
-typedef struct
-{
-    char **papszFiles;
-    int nCount;
-    int i;
-    char* pszPath;
-    char* pszDisplayedPath;
-} VSIReadDirRecursiveTask;
-
 /**
  * \brief Read names in a directory recursively.
  *
@@ -171,7 +162,7 @@ typedef struct
  * It returns a list of strings containing the names of files and directories
  * in this directory and all subdirectories.  The resulting string list becomes
  * the responsibility of the application and should be freed with CSLDestroy()
- *  when no longer needed.
+ * when no longer needed.
  *
  * Note that no error is issued via CPLError() if the directory path is
  * invalid, though NULL is returned.
@@ -188,136 +179,24 @@ typedef struct
 
 char **VSIReadDirRecursive( const char *pszPathIn )
 {
+    const char* const apszOptions[] = { "NAME_AND_TYPE_ONLY=YES", nullptr };
+    VSIDIR* psDir = VSIOpenDir(pszPathIn, -1, apszOptions);
+    if( !psDir )
+        return nullptr;
     CPLStringList oFiles;
-    char **papszFiles = nullptr;
-    VSIStatBufL psStatBuf;
-    CPLString osTemp1;
-    CPLString osTemp2;
-    int i = 0;
-    int nCount = -1;
-
-    std::vector<VSIReadDirRecursiveTask> aoStack;
-    char* pszPath = CPLStrdup(pszPathIn);
-    char* pszDisplayedPath = nullptr;
-
-    while( true )
+    while( auto psEntry = VSIGetNextDirEntry(psDir) )
     {
-        if( nCount < 0 )
+        if( VSI_ISDIR(psEntry->nMode) && psEntry->pszName[0] &&
+            psEntry->pszName[strlen(psEntry->pszName)-1] != '/' )
         {
-            // Get listing.
-            papszFiles = VSIReadDir( pszPath );
-
-            // Get files and directories inside listing.
-            nCount = papszFiles ? CSLCount( papszFiles ) : 0;
-            i = 0;
+            oFiles.AddString((std::string(psEntry->pszName) + '/').c_str());
         }
-
-        for( ; i < nCount; i++ )
+        else
         {
-            // Do not recurse up the tree.
-            if( EQUAL(".", papszFiles[i]) || EQUAL("..", papszFiles[i]) )
-              continue;
-
-            // Build complete file name for stat.
-            osTemp1.clear();
-            osTemp1.append( pszPath );
-            if( !osTemp1.empty() && osTemp1.back() != '/' )
-                osTemp1.append( "/" );
-            osTemp1.append( papszFiles[i] );
-
-            // If is file, add it.
-            if( VSIStatL( osTemp1.c_str(), &psStatBuf ) != 0 )
-                continue;
-
-            if( VSI_ISREG( psStatBuf.st_mode ) )
-            {
-                if( pszDisplayedPath )
-                {
-                    osTemp1.clear();
-                    osTemp1.append( pszDisplayedPath );
-                    if( !osTemp1.empty() && osTemp1.back() != '/' )
-                        osTemp1.append( "/" );
-                    osTemp1.append( papszFiles[i] );
-                    oFiles.AddString( osTemp1 );
-                }
-                else
-                    oFiles.AddString( papszFiles[i] );
-            }
-            else if( VSI_ISDIR( psStatBuf.st_mode ) )
-            {
-                // Add directory entry.
-                osTemp2.clear();
-                if( pszDisplayedPath )
-                {
-                    osTemp2.append( pszDisplayedPath );
-                    osTemp2.append( "/" );
-                }
-                osTemp2.append( papszFiles[i] );
-                if( !osTemp2.empty() && osTemp2.back() != '/' )
-                    osTemp2.append( "/" );
-                oFiles.AddString( osTemp2.c_str() );
-
-                VSIReadDirRecursiveTask sTask;
-                sTask.papszFiles = papszFiles;
-                sTask.nCount = nCount;
-                sTask.i = i;
-                sTask.pszPath = CPLStrdup(pszPath);
-                sTask.pszDisplayedPath =
-                    pszDisplayedPath ? CPLStrdup(pszDisplayedPath) : nullptr;
-                aoStack.push_back(sTask);
-
-                CPLFree(pszPath);
-                pszPath = CPLStrdup( osTemp1.c_str() );
-
-                char* pszDisplayedPathNew = nullptr;
-                if( pszDisplayedPath )
-                {
-                    pszDisplayedPathNew =
-                        CPLStrdup(
-                            CPLSPrintf("%s/%s",
-                                       pszDisplayedPath, papszFiles[i]));
-                }
-                else
-                {
-                    pszDisplayedPathNew = CPLStrdup( papszFiles[i] );
-                }
-                CPLFree(pszDisplayedPath);
-                pszDisplayedPath = pszDisplayedPathNew;
-
-                i = 0;
-                papszFiles = nullptr;
-                nCount = -1;
-
-                break;
-            }
-        }
-
-        if( nCount >= 0 )
-        {
-            CSLDestroy( papszFiles );
-
-            if( !aoStack.empty() )
-            {
-                const int iLast = static_cast<int>(aoStack.size()) - 1;
-                CPLFree(pszPath);
-                CPLFree(pszDisplayedPath);
-                nCount = aoStack[iLast].nCount;
-                papszFiles = aoStack[iLast].papszFiles;
-                i = aoStack[iLast].i + 1;
-                pszPath = aoStack[iLast].pszPath;
-                pszDisplayedPath = aoStack[iLast].pszDisplayedPath;
-
-                aoStack.resize(iLast);
-            }
-            else
-            {
-                break;
-            }
+            oFiles.AddString(psEntry->pszName);
         }
     }
-
-    CPLFree(pszPath);
-    CPLFree(pszDisplayedPath);
+    VSICloseDir(psDir);
 
     return oFiles.StealList();
 }
@@ -361,7 +240,18 @@ char **CPLReadDir( const char *pszPath )
  * @param nRecurseDepth 0 means do not recurse in subdirectories, 1 means
  * recurse only in the first level of subdirectories, etc. -1 means unlimited
  * recursion level
- * @param papszOptions NULL terminated list of options, or NULL.
+ * @param papszOptions NULL terminated list of options, or NULL. The following
+ * options are implemented:
+ * <ul>
+ * <li>PREFIX=string: (GDAL >= 3.4) Filter to select filenames only starting
+ *     with the specified prefix. Implemented efficiently for /vsis3/, /vsigs/,
+ *     and /vsiaz/ (but not /vsiadls/)
+ * </li>
+ * <li>NAME_AND_TYPE_ONLY=YES/NO: (GDAL >= 3.4) Defaults to NO. If set to YES,
+ *     only the pszName and nMode members of VSIDIR are guaranteed to be set.
+ *     This is implemented efficiently for the Unix virtual file system.
+ * </li>
+ * </ul>
  *
  * @return a handle, or NULL in case of error
  * @since GDAL 2.4
@@ -542,6 +432,9 @@ int VSIUnlink( const char * pszFilename )
  *
  * All files should belong to the same file system handler.
  *
+ * This is implemented efficiently for /vsis3/ and /vsigs/ (provided for /vsigs/
+ * that OAuth2 authentication is used).
+ *
  * @param papszFiles NULL terminated list of files. UTF-8 encoded.
  *
  * @return an array of size CSLCount(papszFiles), whose values are TRUE or FALSE
@@ -712,6 +605,33 @@ int VSISync( const char* pszSource, const char* pszTarget,
                 TRUE : FALSE;
 }
 
+/************************************************************************/
+/*                         VSIAbortOngoingUploads()                     */
+/************************************************************************/
+
+/**
+ * \brief Abort ongoing multi-part uploads.
+ *
+ * Abort ongoing multi-part uploads on AWS S3 and Google Cloud Storage. This
+ * can be used in case a process doing such uploads was killed in a unclean way.
+ *
+ * Without effect on other virtual file systems.
+ *
+ * @param pszFilename filename or prefix of a directory into which multipart uploads must be
+ *                    aborted. This can be the root directory of a bucket.  UTF-8 encoded.
+ *
+ * @return TRUE on success or FALSE on an error.
+ * @since GDAL 3.4
+ */
+
+int VSIAbortPendingUploads( const char* pszFilename )
+{
+    VSIFilesystemHandler *poFSHandler =
+        VSIFileManager::GetHandler( pszFilename );
+
+    return poFSHandler->AbortPendingUploads( pszFilename );
+}
+
 /************************************************************************/
 /*                              VSIRmdir()                              */
 /************************************************************************/
@@ -752,6 +672,8 @@ int VSIRmdir( const char * pszDirname )
  *
  * Starting with GDAL 3.1, /vsis3/ has an efficient implementation of this
  * function.
+ * Starting with GDAL 3.4, /vsigs/ has an efficient implementation of this
+ * function, provided that OAuth2 authentication is used.
  *
  * @return 0 on success or -1 on an error.
  * @since GDAL 2.3
@@ -825,8 +747,8 @@ int VSIStatL( const char * pszFilename, VSIStatBufL *psStatBuf )
  * UTF-8 encoded.
  * @param psStatBuf the structure to load with information.
  * @param nFlags 0 to get all information, or VSI_STAT_EXISTS_FLAG,
- *                  VSI_STAT_NATURE_FLAG or VSI_STAT_SIZE_FLAG, or a
- *                  combination of those to get partial info.
+ *                 VSI_STAT_NATURE_FLAG, VSI_STAT_SIZE_FLAG, VSI_STAT_SET_ERROR_FLAG,
+ *                 VSI_STAT_CACHE_ONLY or a combination of those to get partial info.
  *
  * @return 0 on success or -1 on an error.
  *
@@ -875,7 +797,7 @@ int VSIStatExL( const char * pszFilename, VSIStatBufL *psStatBuf, int nFlags )
  * @param pszDomain Metadata domain to query. Depends on the file system.
  * The following are supported:
  * <ul>
- * <li>HEADERS: to get HTTP headers for network-like filesystems (/vsicurl/, /vsis3/, etc)</li>
+ * <li>HEADERS: to get HTTP headers for network-like filesystems (/vsicurl/, /vsis3/, /vsgis/, etc)</li>
  * <li>TAGS:
  *    <ul>
  *      <li>/vsis3/: to get S3 Object tagging information</li>
@@ -883,7 +805,9 @@ int VSIStatExL( const char * pszFilename, VSIStatBufL *psStatBuf, int nFlags )
  *    </ul>
  * </li>
  * <li>STATUS: specific to /vsiadls/: returns all system defined properties for a path (seems in practice to be a subset of HEADERS)</li>
- * <li>ACL: specific to /vsiadls/: returns the access control list for a path.</li>
+ * <li>ACL: specific to /vsiadls/ and /vsigs/: returns the access control list for a path.
+ *     For /vsigs/, a single XML=xml_content string is returned. Refer to https://cloud.google.com/storage/docs/xml-api/get-object-acls
+ * </li>
  * <li>METADATA: specific to /vsiaz/: to set blob metadata. Refer to https://docs.microsoft.com/en-us/rest/api/storageservices/get-blob-metadata. Note: this will be a subset of what pszDomain=HEADERS returns</li>
  * </ul>
  * @param papszOptions Unused. Should be set to NULL.
@@ -909,7 +833,7 @@ char** VSIGetFileMetadata( const char * pszFilename, const char* pszDomain,
 /**
  * \brief Set metadata on files.
  *
- * Implemented currently only for /vsis3/, /vsiaz/ and /vsiadls/
+ * Implemented currently only for /vsis3/, /vsigs/, /vsiaz/ and /vsiadls/
  *
  * @param pszFilename the path of the filesystem object to be set.
  * UTF-8 encoded.
@@ -917,7 +841,11 @@ char** VSIGetFileMetadata( const char * pszFilename, const char* pszDomain,
  * @param pszDomain Metadata domain to set. Depends on the file system.
  * The following are supported:
  * <ul>
- * <li>HEADERS: specific to /vsis3/: to set HTTP headers</li>
+ * <li>HEADERS: specific to /vsis3/ and /vsigs/: to set HTTP headers, such as
+ * "Content-Type", or other file system specific header.
+ * For /vsigs/, this also includes: x-goog-meta-{key}={value}. Note that you
+ * should specify all metadata to be set, as existing metadata will be overridden.
+ * </li>
  * <li>TAGS: Content of papszMetadata should be KEY=VALUE pairs.
  *    <ul>
  *      <li>/vsis3/: to set S3 Object tagging information</li>
@@ -930,7 +858,10 @@ char** VSIGetFileMetadata( const char * pszFilename, const char* pszDomain,
  *      <li>to /vsiadls/: to set properties. Refer to https://docs.microsoft.com/en-us/rest/api/storageservices/datalakestoragegen2/path/update for headers valid for action=setProperties.</li>
  *    </ul>
  * </li>
- * <li>ACL: specific to /vsiadls/: to set access control list. Refer to https://docs.microsoft.com/en-us/rest/api/storageservices/datalakestoragegen2/path/update for headers valid for action=setAccessControl or setAccessControlRecursive. In setAccessControlRecursive, x-ms-acl must be specified in papszMetadata</li>
+ * <li>ACL: specific to /vsiadls/ and /vsigs/: to set access control list.
+ * For /vsiadls/, refer to https://docs.microsoft.com/en-us/rest/api/storageservices/datalakestoragegen2/path/update for headers valid for action=setAccessControl or setAccessControlRecursive. In setAccessControlRecursive, x-ms-acl must be specified in papszMetadata.
+ * For /vsigs/, refer to https://cloud.google.com/storage/docs/xml-api/put-object-acls. A single XML=xml_content string should be specified as in papszMetadata.
+ * </li>
  * <li>METADATA: specific to /vsiaz/: to set blob metadata. Refer to https://docs.microsoft.com/en-us/rest/api/storageservices/set-blob-metadata. Content of papszMetadata should be strings in the form x-ms-meta-name=value</li>
  * </ul>
  * @param papszOptions NULL or NULL terminated list of options.
@@ -1054,7 +985,7 @@ int VSIHasOptimizedReadMultiRange( const char* pszPath )
  * Currently only returns a non-NULL value for network-based virtual file systems.
  * For example "/vsis3/bucket/filename" will be expanded as
  * "https://bucket.s3.amazon.com/filename"
- * 
+ *
  * Note that the lifetime of the returned string, is short, and may be
  * invalidated by any following GDAL functions.
  *
@@ -1275,7 +1206,7 @@ bool VSIFilesystemHandler::Sync( const char* pszSource, const char* pszTarget,
         {
             osTarget = CPLFormFilename(osTarget,
                                        CPLGetFilename(pszSource), nullptr);
-            bTargetIsFile = VSIStatL(osTarget, &sTarget) == 0 && 
+            bTargetIsFile = VSIStatL(osTarget, &sTarget) == 0 &&
                             !CPL_TO_BOOL(VSI_ISDIR(sTarget.st_mode));
         }
         if( bTargetIsFile )
@@ -1407,6 +1338,7 @@ struct VSIDIRGeneric: public VSIDIR
     VSIDIREntry entry{};
     std::vector<VSIDIRGeneric*> aoStackSubDir{};
     VSIFilesystemHandler* poFS = nullptr;
+    std::string m_osFilterPrefix{};
 
     explicit VSIDIRGeneric(VSIFilesystemHandler* poFSIn): poFS(poFSIn) {}
     ~VSIDIRGeneric();
@@ -1439,7 +1371,7 @@ VSIDIRGeneric::~VSIDIRGeneric()
 
 VSIDIR* VSIFilesystemHandler::OpenDir( const char *pszPath,
                                        int nRecurseDepth,
-                                       const char* const *)
+                                       const char* const * papszOptions)
 {
     char** papszContent = VSIReadDir(pszPath);
     VSIStatBufL sStatL;
@@ -1452,6 +1384,7 @@ VSIDIR* VSIFilesystemHandler::OpenDir( const char *pszPath,
     dir->osRootPath = pszPath;
     dir->nRecurseDepth = nRecurseDepth;
     dir->papszContent = papszContent;
+    dir->m_osFilterPrefix = CSLFetchNameValueDef(papszOptions, "PREFIX", "");
     return dir;
 }
 
@@ -1461,6 +1394,7 @@ VSIDIR* VSIFilesystemHandler::OpenDir( const char *pszPath,
 
 const VSIDIREntry* VSIDIRGeneric::NextDirEntry()
 {
+begin:
     if( VSI_ISDIR(entry.nMode) && nRecurseDepth != 0 )
     {
         CPLString osCurFile(osRootPath);
@@ -1474,6 +1408,7 @@ const VSIDIREntry* VSIDIRGeneric::NextDirEntry()
         {
             subdir->osRootPath = osRootPath;
             subdir->osBasePath = entry.pszName;
+            subdir->m_osFilterPrefix = m_osFilterPrefix;
             aoStackSubDir.push_back(subdir);
         }
         entry.nMode = 0;
@@ -1511,41 +1446,69 @@ const VSIDIREntry* VSIDIRGeneric::NextDirEntry()
         }
         else
         {
+            CPLFree(entry.pszName);
+            CPLString osName(osBasePath);
+            if( !osName.empty() )
+                osName += '/';
+            osName += papszContent[nPos];
+            nPos ++;
+
+            entry.pszName = CPLStrdup(osName);
+            entry.nMode = 0;
+            CPLString osCurFile(osRootPath);
+            if( !osCurFile.empty() )
+                osCurFile += '/';
+            osCurFile += entry.pszName;
+
+            const auto StatFile = [&osCurFile, this]()
+            {
+                VSIStatBufL sStatL;
+                if( VSIStatL(osCurFile, &sStatL) == 0 )
+                {
+                    entry.nMode = sStatL.st_mode;
+                    entry.nSize = sStatL.st_size;
+                    entry.nMTime = sStatL.st_mtime;
+                    entry.bModeKnown = true;
+                    entry.bSizeKnown = true;
+                    entry.bMTimeKnown = true;
+                }
+                else
+                {
+                    entry.nMode = 0;
+                    entry.nSize = 0;
+                    entry.nMTime = 0;
+                    entry.bModeKnown = false;
+                    entry.bSizeKnown = false;
+                    entry.bMTimeKnown = false;
+                }
+            };
+
+            if( !m_osFilterPrefix.empty() &&
+                m_osFilterPrefix.size() > osName.size() )
+            {
+                if( STARTS_WITH(m_osFilterPrefix.c_str(), osName.c_str()) &&
+                    m_osFilterPrefix[osName.size()] == '/' )
+                {
+                    StatFile();
+                    if( VSI_ISDIR(entry.nMode) )
+                    {
+                        goto begin;
+                    }
+                }
+                continue;
+            }
+            if( !m_osFilterPrefix.empty() &&
+                !STARTS_WITH(osName.c_str(), m_osFilterPrefix.c_str()) )
+            {
+                continue;
+            }
+
+            StatFile();
+
             break;
         }
     }
 
-    CPLFree(entry.pszName);
-    CPLString osName(osBasePath);
-    if( !osName.empty() )
-        osName += '/';
-    osName += papszContent[nPos];
-    entry.pszName = CPLStrdup(osName);
-    VSIStatBufL sStatL;
-    CPLString osCurFile(osRootPath);
-    if( !osCurFile.empty() )
-        osCurFile += '/';
-    osCurFile += entry.pszName;
-    if( VSIStatL(osCurFile, &sStatL) == 0 )
-    {
-        entry.nMode = sStatL.st_mode;
-        entry.nSize = sStatL.st_size;
-        entry.nMTime = sStatL.st_mtime;
-        entry.bModeKnown = true;
-        entry.bSizeKnown = true;
-        entry.bMTimeKnown = true;
-    }
-    else
-    {
-        entry.nMode = 0;
-        entry.nSize = 0;
-        entry.nMTime = 0;
-        entry.bModeKnown = false;
-        entry.bSizeKnown = false;
-        entry.bMTimeKnown = false;
-    }
-    nPos ++;
-
     return &(entry);
 }
 
